/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/


/***********************************************************************
MultiCam sample program - picolo_video

Video is acquired and then displayed on screen. The board must be 
one of the Picolo series frame grabber.  Check the board location in PCI 
slot and make sure set it to parameter "MC_DriverIndex".

Press "ESC" key to close the application.

The application works with PAL camera.  IF you have NTSC, set the 
parameter MC_Camera to MC_Camera_CAMERA_NTSC accordingly.
See InitializeMultiCam function

The program uses automatic memory allocation and works with both 32 and 
64 bit systems.
***********************************************************************/


#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>

#include "/usr/local/euresys/multicam/include/multicam.h"

#ifndef BYTE
#define BYTE unsigned char
#endif

SDL_Surface* screen = NULL;


BOOL error; // Flagged on acquisition error

int sizeX;  // Width of the acquired image
int sizeY;  // Height of the acquired image
volatile bool channelactive ; // track channel true activity

// check if <esc> was hit to exit application
BOOL CheckForQuit()
{
	SDL_Event event;

	// Poll for events, and handle the ones we care about.
	while (SDL_PollEvent(&event)) {
		switch (event.type) {
		case SDL_KEYUP:
			// If escape is pressed, return (and thus, quit)
			if (event.key.keysym.sym == SDLK_ESCAPE)
				return TRUE;
			break;
		case SDL_QUIT:
			return TRUE;
		}
	}

	return FALSE;
}

// Draw acquired image to screen
void UpdateSurface(BYTE *pImage)
{
	// Check if we need to lock the screen
	if (SDL_MUSTLOCK(screen))
		SDL_LockSurface(screen);

	// Copy data from acquisition buffer to screen buffer
	memcpy(screen->pixels, pImage, sizeX* sizeY*4); // 32 bits per pixel = 4 bytes per pixel

	// Does the screen need unlocking
	if (SDL_MUSTLOCK(screen))
		SDL_UnlockSurface(screen);

	// Bit image
	SDL_Flip(screen);
}

//  Callback function registered to our channel. 
//    Called at the asynchronous signals.
void McCallback(PMCCALLBACKINFO CallBackInfo)
{
	MCHANDLE hSurface;
	BYTE *pImage;

	switch(CallBackInfo->Signal)
	{
	case MC_SIG_SURFACE_PROCESSING:
		hSurface = (MCHANDLE) CallBackInfo->SignalInfo;
		McGetParamPtr(hSurface,MC_SurfaceAddr,(PVOID*)&pImage);
		UpdateSurface(pImage);
		break;
	case MC_SIG_ACQUISITION_FAILURE:
		fprintf(stderr, "Acquisition Failure. Is a video signal connected ?\n");
		error = TRUE;
		break;
	case MC_SIG_END_CHANNEL_ACTIVITY:
		channelactive = false;
		break;
	default:
		fprintf(stderr,"Signal not handled:%d", CallBackInfo->Signal);
		error = TRUE;
		break;
	}
}


//  Create a channel and set its parameters
int InitializeMultiCam( MCHANDLE *hc)
{
	MCSTATUS status = 0;
	MCHANDLE& hChannel = *hc;

	int i;

	sizeX=0;
	sizeY=0;

	//  Create a channel.
	status = McCreate(MC_CHANNEL, &hChannel);
	if (status != MC_OK) goto Finalize;

	// Link the channel to a board. Here we take the first board.
	status = McSetParamInt(hChannel, MC_DriverIndex, 0);
	if (status != MC_OK) goto Finalize;

	// Select the video connector. Here the first video connector.
	status = McSetParamInt(hChannel, MC_Connector, MC_Connector_VID1);
	if (status != MC_OK) goto Finalize;

	// Select the video signal type. if NTSC is needed change MC_Camera_CAMERA_PAL to MC_Camera_CAMERA_NTSC
	status = McSetParamInt(hChannel, MC_Camera, MC_Camera_CAMERA_PAL);
	if (status != MC_OK) goto Finalize;

	// Set the color format.
	status = McSetParamInt(hChannel, MC_ColorFormat, MC_ColorFormat_RGB32);
	if (status != MC_OK) goto Finalize;

	// Retrieve channel size information.
	status= McGetParamInt(hChannel, MC_ImageSizeX, &sizeX);
	if (status != MC_OK) goto Finalize;
	status= McGetParamInt(hChannel, MC_ImageSizeY, &sizeY);
	if (status != MC_OK) goto Finalize;  

	// The number of images to acquire.
	status = McSetParamInt (hChannel, MC_SeqLength_Fr, MC_INDETERMINATE);
	if (status != MC_OK) goto Finalize;

	// Register our Callback function for the MultiCam asynchronous signals.
	status = McRegisterCallback(hChannel, McCallback, NULL);
	if (status != MC_OK) goto Finalize;

	// Enable the signals we need: 
	// MC_SIG_SURFACE_PROCESSING: acquisition done and locked for processing
	// MC_SIG_ACQUISITION_FAILURE: acquisition failed.
	// MC_SIG_END_CHANNEL_ACTIVITY: track channel activity
	status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_SURFACE_PROCESSING, MC_SignalEnable_ON);
	if (status != MC_OK) goto Finalize;

	status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_ACQUISITION_FAILURE, MC_SignalEnable_ON);
	if (status != MC_OK) goto Finalize;

	status = McSetParamInt(hChannel, MC_SignalEnable + MC_SIG_END_CHANNEL_ACTIVITY, MC_SignalEnable_ON);
	if (status != MC_OK) goto Finalize;

	return 0;

Finalize:
	if (status!=MC_OK)
	{
		printf("Acquire Error %d\n",status);
	}

	if (hChannel)
	{
		McDelete(hChannel);
	}

	return -1;

}

// Start the image acquisition, images are processed in the callback
void AcquireImages(MCHANDLE hChannel) 
{
	MCSTATUS status = 0;

	// Start Acquisitions for this channel. 
	status = McSetParamInt(hChannel, MC_ChannelState, MC_ChannelState_ACTIVE);
	if (status != MC_OK) goto Finalize;
	channelactive = true;

	// wait until user press <esc> key
	fprintf(stderr,"Waiting for callbacks\n");
	while (!CheckForQuit() && !error ) 
	{ 
		sleep(1);
	}

	status= McSetParamInt(hChannel, MC_ChannelState, MC_ChannelState_IDLE);
	
	// wait until the channel has performed the last acquisition
	while (channelactive)
		usleep(10);

Finalize:
	if (status!=MC_OK)
	{
		printf("Acquire Error %d\n",status);
	}

	if (hChannel)
	{
		McDelete(hChannel);
	}
}

// Application entry point
int main(int argc, char* argv[])
{
	MCSTATUS status;
	MCHANDLE hChannel = 0;
	error = FALSE;
	channelactive = false;

	// Initialize the MultiCam driver. 
	status = McOpenDriver(NULL);
	if(status != MC_OK)
	{
		fprintf(stderr,"Error - Fail to initialize MultiCam\n");
		return -1;
	}

	// Initialize MultiCam channel
	if (InitializeMultiCam(&hChannel))
		return -1;

	// Initialize SDL library and create window
	SDL_Init( SDL_INIT_VIDEO );
	screen = SDL_SetVideoMode(sizeX, sizeY, 32, SDL_SWSURFACE | SDL_DOUBLEBUF);

	// Acquire images
	fprintf(stderr,"Press <Esc> key to exit\n");
	AcquireImages(hChannel);

	// Quit SDL
	SDL_Quit();

	// Close the MultiCam driver.
	McCloseDriver();

	return 0;
}

